/* FILE: fixedzeeman.cc           -*-Mode: c++-*-
 *
 * Fixed (in time) Zeeman energy/field, derived from Oxs_Energy class.
 *
 */

#include <memory>   // auto_ptr template

#include "oc.h"
#include "nb.h"
#include "threevector.h"
#include "energy.h"
#include "simstate.h"
#include "mesh.h"
#include "meshvalue.h"
#include "vectorfieldinit.h"
#include "fixedzeeman.h"

// Oxs_Ext registration support
OXS_EXT_REGISTER(Oxs_FixedZeeman);

/* End includes */


// Constructor
Oxs_FixedZeeman::Oxs_FixedZeeman(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  Tcl_Interp* safe_interp, // Safe interpreter
  const char* argstr)   // MIF input block parameters
  : Oxs_Energy(name,newdtr,safe_interp,argstr), mesh_id(0)
{
  // Process arguments
  // Note: Stroustrup specifies an auto_ptr .reset() member function,
  // but that is apparently not supported by some common STL's.  As
  // a workaround, create a temporary auto_ptr, and use operator=().
  auto_ptr<Oxs_VectorFieldInit>
    bar(NewVectorFieldInit(safe_interp,"field"));
  fixedfield_init = bar;
  VerifyAllInitArgsUsed();
}

void Oxs_FixedZeeman::GetEnergyAndField
(const Oxs_SimState& state,
 Oxs_MeshValue<REAL8m>& energy,
 Oxs_MeshValue<ThreeVector>& field
 ) const
{
  UINT4m size = state.mesh->Size();
  if(size<1) return; // Nothing to do
  if(mesh_id != state.mesh->Id()) {
    // This is either the first pass through, or else mesh
    // has changed.
    mesh_id = 0;
    fixedfield_init->FillMeshValue(state.mesh,fixedfield);
    mesh_id = state.mesh->Id();
  }

  const Oxs_MeshValue<ThreeVector>& spin = state.spin;
  const Oxs_MeshValue<REAL8m>& Ms = *(state.Ms);
  UINT4m i=0;
  do {
    field[i] = fixedfield[i];
    energy[i] = -MU0*Ms[i]*(fixedfield[i]*spin[i]);
    ++i;
  } while(i<size);
}



