/* FILE: uzeeman.h            -*-Mode: c++-*-
 *
 * Uniform Zeeman (applied field) energy, derived from Oxs_Energy class.
 *
 */

#ifndef _OXS_UZEEMAN
#define _OXS_UZEEMAN

#include <vector>
#include "threevector.h"
#include "output.h"
#include "energy.h"

/* End includes */

class Oxs_UZeeman:public Oxs_Energy {
private:
  vector<ThreeVector> Happ;  // Indexed off of state stage_number
  ThreeVector GetAppliedField(UINT4m stage_number) const;
  /// Utility function. Indexes Happ off of stage_number with
  /// proper range behavior.

  // Supplied outputs, in addition to those provided by Oxs_Energy.
  Oxs_ScalarOutput<Oxs_UZeeman> Bapp_output;
  Oxs_ScalarOutput<Oxs_UZeeman> Bappx_output;
  Oxs_ScalarOutput<Oxs_UZeeman> Bappy_output;
  Oxs_ScalarOutput<Oxs_UZeeman> Bappz_output;
  void Fill__Bapp_output(const Oxs_SimState& state);

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_UZeeman(const char* name,     // Child instance id
	      Oxs_Director* newdtr, // App director
              Tcl_Interp* safe_interp, // Safe interpreter
	      const char* argstr);  // MIF input block parameters
  virtual ~Oxs_UZeeman();

  virtual void GetEnergyAndField(const Oxs_SimState& state,
                                 Oxs_MeshValue<REAL8m>& energy,
                                 Oxs_MeshValue<ThreeVector>& field
                                 ) const;
};


#endif // _OXS_UZEEMAN
